@@ -438,31 +438,6 @@ static void xdebug_analyse_oparray(zend_op_array *opa, xdebug_set *set, xdebug_b
 	}
 }
 
-void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa)
-{
-	int closure = 0;
-
-	memset(tmp, 0, sizeof(xdebug_func));
-
-	if (opa->function_name) {
-		if (xdebug_function_name_is_closure(STR_NAME_VAL(opa->function_name))) {
-			tmp->function = xdebug_wrap_closure_location_around_function_name(opa, STR_NAME_VAL(opa->function_name));
-			closure = 1;
-		} else {
-			tmp->function = xdstrdup(STR_NAME_VAL(opa->function_name));
-		}
-	} else {
-		tmp->function = xdstrdup("{main}");
-	}
-
-	if (opa->scope && !closure) {
-		tmp->type = XFUNC_MEMBER;
-		tmp->class = xdstrdup(STR_NAME_VAL(opa->scope->name));
-	} else {
-		tmp->type = XFUNC_NORMAL;
-	}
-}
-
 static void prefill_from_oparray(char *filename, zend_op_array *op_array)
 {
 	unsigned int i;
@@ -1023,6 +998,7 @@ void xdebug_coverage_minit(INIT_FUNC_ARGS)
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_STMT);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR_NO_REF);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR_NO_REF_EX);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_REF);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAL);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAL_EX);
@@ -1030,6 +1006,7 @@ void xdebug_coverage_minit(INIT_FUNC_ARGS)
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_NEW);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_FCALL_BEGIN);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_INIT_METHOD_CALL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_INIT_STATIC_METHOD_CALL);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_INIT_FCALL);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_CATCH);
 		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL);
@@ -1133,6 +1110,7 @@ void xdebug_coverage_mshutdown(void)
 		zend_set_user_opcode_handler(ZEND_EXT_STMT, NULL);
 		zend_set_user_opcode_handler(ZEND_SEND_VAR, NULL);
 		zend_set_user_opcode_handler(ZEND_SEND_VAR_NO_REF, NULL);
+		zend_set_user_opcode_handler(ZEND_SEND_VAR_NO_REF_EX, NULL);
 		zend_set_user_opcode_handler(ZEND_SEND_REF, NULL);
 		zend_set_user_opcode_handler(ZEND_SEND_VAL, NULL);
 		zend_set_user_opcode_handler(ZEND_SEND_VAL_EX, NULL);
@@ -1140,6 +1118,7 @@ void xdebug_coverage_mshutdown(void)
 		zend_set_user_opcode_handler(ZEND_NEW, NULL);
 		zend_set_user_opcode_handler(ZEND_EXT_FCALL_BEGIN, NULL);
 		zend_set_user_opcode_handler(ZEND_INIT_METHOD_CALL, NULL);
+		zend_set_user_opcode_handler(ZEND_INIT_STATIC_METHOD_CALL, NULL);
 		zend_set_user_opcode_handler(ZEND_INIT_FCALL, NULL);
 		zend_set_user_opcode_handler(ZEND_CATCH, NULL);
 		zend_set_user_opcode_handler(ZEND_BOOL, NULL);
